from textual.widgets import Header, Footer, OptionList
from textual.widgets.option_list import Option
# > Typing
from typing import Optional, Callable

class DataOption(Option):
    def __init__(
        self,
        text: str,
        selected: bool=False,
        id: Optional[str]=None,
        disable: bool=False,
        **data
    ) -> None:
        super().__init__(text, id=id, disabled=disable)
        self.group = ""
        self.selected = selected
        self.data = data

class DataOptionList(OptionList):
    def __init__(
        self,
        *content: DataOption,
        group: Optional[str]="",
        after_selected: Callable[[DataOption], None]=lambda option: None
    ) -> None:
        super().__init__()
        self.group = group
        self.content = content
        self.after_selected = after_selected
    
    def on_mount(self):
        for index, option in enumerate(self.content):
            self.add_option(option)
            if isinstance(option, DataOption):
                option.group = self.group
                if option.selected:
                    self.highlighted = index
    
    async def on_option_list_option_selected(self, event: OptionList.OptionSelected) -> None:
        if isinstance(event.option, DataOption):
            if self.group == event.option.group:
                await self.after_selected(event.option)



from pydantic import BaseModel
# > Local Import's
from typing import Optional

# ! Plugin Info Class
class PluginInfo(BaseModel):
    name: str
    name_id: str
    version: str
    author: str
    description: Optional[str]=None
    url: Optional[str]=None

# ! Plugin Base Class
class PluginBase:
    def __init_repr__(self) -> str:
        return f"[green]{self.info.name}[/] ({repr(self.info.name_id)}) [cyan]v{self.info.version}[/cyan] [yellow]is initialized[/yellow]!"
    
    def __init__(self, app, pl, info: PluginInfo) -> None:
        self.app = app
        self.pl = pl
        self.info = info
        
        self.app.info(self.__init_repr__())

    def on_init(self): pass
    def on_run(self): pass
    async def on_compose(self): pass
    async def on_quit(self): pass



class PluginNotExistsError(Exception):
    def __init__(self, name_id: str) -> None:
        super().__init__()
        self.args = (f"The {repr(name_id)} plugin was not found.",)

class IsPluginDirectoryError(Exception):
    def __init__(self, dirpath: str) -> None:
        super().__init__()
        self.args = (f"There is already a plugin in this directory: {repr(dirpath)}.",)